use crate::types::abi::registry::Registry;
use crate::Farcaster;

impl Farcaster {
    /// ## Get 1 of the two Registry ABIs
    /// 
    /// Get either the ID or Name registry ABI's (for the FC V2 smart contracts on Goerli)
    /// 
    /// ```
    /// pub fn get_registry_abi(registry: Registry) -> Option<&'static str>
    /// ```
    /// 
    /// ## Arguments
    /// 
    /// * `registry`: types::abi::registry::Registry
    ///     - Registry is an enum which has two types:
    ///         - ID (Represents the ID registry ABI)
    ///         - NAME (Represents the Name registry ABI)
    /// 
    /// ## Return Type
    /// ``-> Option<&'static str>``
    ///     - Returns an Optional String 
    /// 
    /// ## Usage
    /// ```
    /// use farcaster_rs::{Farcaster, types::abi::registry::Registry};
    /// 
    /// let registry = Farcaster::get_registry_abi(Registry::ID).unwrap();
    /// ```
    pub fn get_registry_abi(registry: Registry) -> Option<&'static str> {
        match registry {
            Registry::ID => {
                let id_registry_abi = r#"[
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "_forwarder",
                          "type": "address"
                        }
                      ],
                      "stateMutability": "nonpayable",
                      "type": "constructor"
                    },
                    {
                      "inputs": [],
                      "name": "Escrow",
                      "type": "error"
                    },
                    {
                      "inputs": [],
                      "name": "HasId",
                      "type": "error"
                    },
                    {
                      "inputs": [],
                      "name": "HasNoId",
                      "type": "error"
                    },
                    {
                      "inputs": [],
                      "name": "Invitable",
                      "type": "error"
                    },
                    {
                      "inputs": [],
                      "name": "NoRecovery",
                      "type": "error"
                    },
                    {
                      "inputs": [],
                      "name": "Registrable",
                      "type": "error"
                    },
                    {
                      "inputs": [],
                      "name": "Unauthorized",
                      "type": "error"
                    },
                    {
                      "anonymous": false,
                      "inputs": [
                        {
                          "indexed": true,
                          "internalType": "address",
                          "name": "by",
                          "type": "address"
                        },
                        {
                          "indexed": true,
                          "internalType": "uint256",
                          "name": "id",
                          "type": "uint256"
                        }
                      ],
                      "name": "CancelRecovery",
                      "type": "event"
                    },
                    {
                      "anonymous": false,
                      "inputs": [
                        {
                          "indexed": true,
                          "internalType": "uint256",
                          "name": "id",
                          "type": "uint256"
                        },
                        {
                          "indexed": false,
                          "internalType": "string",
                          "name": "url",
                          "type": "string"
                        }
                      ],
                      "name": "ChangeHome",
                      "type": "event"
                    },
                    {
                      "anonymous": false,
                      "inputs": [
                        {
                          "indexed": true,
                          "internalType": "uint256",
                          "name": "id",
                          "type": "uint256"
                        },
                        {
                          "indexed": true,
                          "internalType": "address",
                          "name": "recovery",
                          "type": "address"
                        }
                      ],
                      "name": "ChangeRecoveryAddress",
                      "type": "event"
                    },
                    {
                      "anonymous": false,
                      "inputs": [
                        {
                          "indexed": true,
                          "internalType": "address",
                          "name": "trustedCaller",
                          "type": "address"
                        }
                      ],
                      "name": "ChangeTrustedCaller",
                      "type": "event"
                    },
                    {
                      "anonymous": false,
                      "inputs": [],
                      "name": "DisableTrustedOnly",
                      "type": "event"
                    },
                    {
                      "anonymous": false,
                      "inputs": [
                        {
                          "indexed": true,
                          "internalType": "address",
                          "name": "previousOwner",
                          "type": "address"
                        },
                        {
                          "indexed": true,
                          "internalType": "address",
                          "name": "newOwner",
                          "type": "address"
                        }
                      ],
                      "name": "OwnershipTransferred",
                      "type": "event"
                    },
                    {
                      "anonymous": false,
                      "inputs": [
                        {
                          "indexed": true,
                          "internalType": "address",
                          "name": "to",
                          "type": "address"
                        },
                        {
                          "indexed": true,
                          "internalType": "uint256",
                          "name": "id",
                          "type": "uint256"
                        },
                        {
                          "indexed": false,
                          "internalType": "address",
                          "name": "recovery",
                          "type": "address"
                        },
                        {
                          "indexed": false,
                          "internalType": "string",
                          "name": "url",
                          "type": "string"
                        }
                      ],
                      "name": "Register",
                      "type": "event"
                    },
                    {
                      "anonymous": false,
                      "inputs": [
                        {
                          "indexed": true,
                          "internalType": "address",
                          "name": "from",
                          "type": "address"
                        },
                        {
                          "indexed": true,
                          "internalType": "address",
                          "name": "to",
                          "type": "address"
                        },
                        {
                          "indexed": true,
                          "internalType": "uint256",
                          "name": "id",
                          "type": "uint256"
                        }
                      ],
                      "name": "RequestRecovery",
                      "type": "event"
                    },
                    {
                      "anonymous": false,
                      "inputs": [
                        {
                          "indexed": true,
                          "internalType": "address",
                          "name": "from",
                          "type": "address"
                        },
                        {
                          "indexed": true,
                          "internalType": "address",
                          "name": "to",
                          "type": "address"
                        },
                        {
                          "indexed": true,
                          "internalType": "uint256",
                          "name": "id",
                          "type": "uint256"
                        }
                      ],
                      "name": "Transfer",
                      "type": "event"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "from",
                          "type": "address"
                        }
                      ],
                      "name": "cancelRecovery",
                      "outputs": [],
                      "stateMutability": "payable",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "string",
                          "name": "url",
                          "type": "string"
                        }
                      ],
                      "name": "changeHome",
                      "outputs": [],
                      "stateMutability": "payable",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "recovery",
                          "type": "address"
                        }
                      ],
                      "name": "changeRecoveryAddress",
                      "outputs": [],
                      "stateMutability": "payable",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "_trustedCaller",
                          "type": "address"
                        }
                      ],
                      "name": "changeTrustedCaller",
                      "outputs": [],
                      "stateMutability": "payable",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "from",
                          "type": "address"
                        }
                      ],
                      "name": "completeRecovery",
                      "outputs": [],
                      "stateMutability": "payable",
                      "type": "function"
                    },
                    {
                      "inputs": [],
                      "name": "completeTransferOwnership",
                      "outputs": [],
                      "stateMutability": "nonpayable",
                      "type": "function"
                    },
                    {
                      "inputs": [],
                      "name": "disableTrustedOnly",
                      "outputs": [],
                      "stateMutability": "payable",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "",
                          "type": "address"
                        }
                      ],
                      "name": "idOf",
                      "outputs": [
                        {
                          "internalType": "uint256",
                          "name": "",
                          "type": "uint256"
                        }
                      ],
                      "stateMutability": "view",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "forwarder",
                          "type": "address"
                        }
                      ],
                      "name": "isTrustedForwarder",
                      "outputs": [
                        {
                          "internalType": "bool",
                          "name": "",
                          "type": "bool"
                        }
                      ],
                      "stateMutability": "view",
                      "type": "function"
                    },
                    {
                      "inputs": [],
                      "name": "owner",
                      "outputs": [
                        {
                          "internalType": "address",
                          "name": "",
                          "type": "address"
                        }
                      ],
                      "stateMutability": "view",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "to",
                          "type": "address"
                        },
                        {
                          "internalType": "address",
                          "name": "recovery",
                          "type": "address"
                        },
                        {
                          "internalType": "string",
                          "name": "url",
                          "type": "string"
                        }
                      ],
                      "name": "register",
                      "outputs": [],
                      "stateMutability": "payable",
                      "type": "function"
                    },
                    {
                      "inputs": [],
                      "name": "renounceOwnership",
                      "outputs": [],
                      "stateMutability": "nonpayable",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "from",
                          "type": "address"
                        },
                        {
                          "internalType": "address",
                          "name": "to",
                          "type": "address"
                        }
                      ],
                      "name": "requestRecovery",
                      "outputs": [],
                      "stateMutability": "payable",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "newOwner",
                          "type": "address"
                        }
                      ],
                      "name": "requestTransferOwnership",
                      "outputs": [],
                      "stateMutability": "nonpayable",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "to",
                          "type": "address"
                        }
                      ],
                      "name": "transfer",
                      "outputs": [],
                      "stateMutability": "payable",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "",
                          "type": "address"
                        }
                      ],
                      "name": "transferOwnership",
                      "outputs": [],
                      "stateMutability": "view",
                      "type": "function"
                    },
                    {
                      "inputs": [
                        {
                          "internalType": "address",
                          "name": "to",
                          "type": "address"
                        },
                        {
                          "internalType": "address",
                          "name": "recovery",
                          "type": "address"
                        },
                        {
                          "internalType": "string",
                          "name": "url",
                          "type": "string"
                        }
                      ],
                      "name": "trustedRegister",
                      "outputs": [],
                      "stateMutability": "payable",
                      "type": "function"
                    }
                  ]"#;
                Some(id_registry_abi)
            }
            Registry::NAME => {
                let name_registry_abi = r#"[
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "_forwarder",
                                "type": "address"
                            }
                        ],
                        "stateMutability": "nonpayable",
                        "type": "constructor"
                    },
                    {
                        "inputs": [],
                        "name": "CallFailed",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "CommitReplay",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "Escrow",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "Expired",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "InsufficientFunds",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "InvalidCommit",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "InvalidName",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "InvalidRecovery",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "Invitable",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "NoRecovery",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "NotAdmin",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "NotBiddable",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "NotInvitable",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "NotModerator",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "NotOperator",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "NotRenewable",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "NotTreasurer",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "Registered",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "Registrable",
                        "type": "error"
                    },
                    {
                        "inputs": [],
                        "name": "Unauthorized",
                        "type": "error"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": false,
                                "internalType": "address",
                                "name": "previousAdmin",
                                "type": "address"
                            },
                            {
                                "indexed": false,
                                "internalType": "address",
                                "name": "newAdmin",
                                "type": "address"
                            }
                        ],
                        "name": "AdminChanged",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "owner",
                                "type": "address"
                            },
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "approved",
                                "type": "address"
                            },
                            {
                                "indexed": true,
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "Approval",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "owner",
                                "type": "address"
                            },
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "operator",
                                "type": "address"
                            },
                            {
                                "indexed": false,
                                "internalType": "bool",
                                "name": "approved",
                                "type": "bool"
                            }
                        ],
                        "name": "ApprovalForAll",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "beacon",
                                "type": "address"
                            }
                        ],
                        "name": "BeaconUpgraded",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "by",
                                "type": "address"
                            },
                            {
                                "indexed": true,
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "CancelRecovery",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": false,
                                "internalType": "uint256",
                                "name": "fee",
                                "type": "uint256"
                            }
                        ],
                        "name": "ChangeFee",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "pool",
                                "type": "address"
                            }
                        ],
                        "name": "ChangePool",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            },
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "recovery",
                                "type": "address"
                            }
                        ],
                        "name": "ChangeRecoveryAddress",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "trustedCaller",
                                "type": "address"
                            }
                        ],
                        "name": "ChangeTrustedCaller",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "vault",
                                "type": "address"
                            }
                        ],
                        "name": "ChangeVault",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [],
                        "name": "DisableTrustedOnly",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": false,
                                "internalType": "uint8",
                                "name": "version",
                                "type": "uint8"
                            }
                        ],
                        "name": "Initialized",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "uint256",
                                "name": "inviterId",
                                "type": "uint256"
                            },
                            {
                                "indexed": true,
                                "internalType": "uint256",
                                "name": "inviteeId",
                                "type": "uint256"
                            },
                            {
                                "indexed": true,
                                "internalType": "bytes16",
                                "name": "fname",
                                "type": "bytes16"
                            }
                        ],
                        "name": "Invite",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": false,
                                "internalType": "address",
                                "name": "account",
                                "type": "address"
                            }
                        ],
                        "name": "Paused",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            },
                            {
                                "indexed": false,
                                "internalType": "uint256",
                                "name": "expiry",
                                "type": "uint256"
                            }
                        ],
                        "name": "Renew",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "from",
                                "type": "address"
                            },
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "to",
                                "type": "address"
                            },
                            {
                                "indexed": true,
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "RequestRecovery",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "bytes32",
                                "name": "role",
                                "type": "bytes32"
                            },
                            {
                                "indexed": true,
                                "internalType": "bytes32",
                                "name": "previousAdminRole",
                                "type": "bytes32"
                            },
                            {
                                "indexed": true,
                                "internalType": "bytes32",
                                "name": "newAdminRole",
                                "type": "bytes32"
                            }
                        ],
                        "name": "RoleAdminChanged",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "bytes32",
                                "name": "role",
                                "type": "bytes32"
                            },
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "account",
                                "type": "address"
                            },
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "sender",
                                "type": "address"
                            }
                        ],
                        "name": "RoleGranted",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "bytes32",
                                "name": "role",
                                "type": "bytes32"
                            },
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "account",
                                "type": "address"
                            },
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "sender",
                                "type": "address"
                            }
                        ],
                        "name": "RoleRevoked",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "from",
                                "type": "address"
                            },
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "to",
                                "type": "address"
                            },
                            {
                                "indexed": true,
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "Transfer",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": false,
                                "internalType": "address",
                                "name": "account",
                                "type": "address"
                            }
                        ],
                        "name": "Unpaused",
                        "type": "event"
                    },
                    {
                        "anonymous": false,
                        "inputs": [
                            {
                                "indexed": true,
                                "internalType": "address",
                                "name": "implementation",
                                "type": "address"
                            }
                        ],
                        "name": "Upgraded",
                        "type": "event"
                    },
                    {
                        "inputs": [],
                        "name": "DEFAULT_ADMIN_ROLE",
                        "outputs": [
                            {
                                "internalType": "bytes32",
                                "name": "",
                                "type": "bytes32"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "to",
                                "type": "address"
                            },
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "approve",
                        "outputs": [],
                        "stateMutability": "nonpayable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "owner",
                                "type": "address"
                            }
                        ],
                        "name": "balanceOf",
                        "outputs": [
                            {
                                "internalType": "uint256",
                                "name": "",
                                "type": "uint256"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "to",
                                "type": "address"
                            },
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            },
                            {
                                "internalType": "address",
                                "name": "recovery",
                                "type": "address"
                            }
                        ],
                        "name": "bid",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "cancelRecovery",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "_fee",
                                "type": "uint256"
                            }
                        ],
                        "name": "changeFee",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "_pool",
                                "type": "address"
                            }
                        ],
                        "name": "changePool",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            },
                            {
                                "internalType": "address",
                                "name": "recovery",
                                "type": "address"
                            }
                        ],
                        "name": "changeRecoveryAddress",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "_trustedCaller",
                                "type": "address"
                            }
                        ],
                        "name": "changeTrustedCaller",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "_vault",
                                "type": "address"
                            }
                        ],
                        "name": "changeVault",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "completeRecovery",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "disableTrustedOnly",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "",
                                "type": "uint256"
                            }
                        ],
                        "name": "expiryOf",
                        "outputs": [
                            {
                                "internalType": "uint256",
                                "name": "",
                                "type": "uint256"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "fee",
                        "outputs": [
                            {
                                "internalType": "uint256",
                                "name": "",
                                "type": "uint256"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "bytes16",
                                "name": "fname",
                                "type": "bytes16"
                            },
                            {
                                "internalType": "address",
                                "name": "to",
                                "type": "address"
                            },
                            {
                                "internalType": "bytes32",
                                "name": "secret",
                                "type": "bytes32"
                            },
                            {
                                "internalType": "address",
                                "name": "recovery",
                                "type": "address"
                            }
                        ],
                        "name": "generateCommit",
                        "outputs": [
                            {
                                "internalType": "bytes32",
                                "name": "commit",
                                "type": "bytes32"
                            }
                        ],
                        "stateMutability": "pure",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "getApproved",
                        "outputs": [
                            {
                                "internalType": "address",
                                "name": "",
                                "type": "address"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "bytes32",
                                "name": "role",
                                "type": "bytes32"
                            }
                        ],
                        "name": "getRoleAdmin",
                        "outputs": [
                            {
                                "internalType": "bytes32",
                                "name": "",
                                "type": "bytes32"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "bytes32",
                                "name": "role",
                                "type": "bytes32"
                            },
                            {
                                "internalType": "address",
                                "name": "account",
                                "type": "address"
                            }
                        ],
                        "name": "grantRole",
                        "outputs": [],
                        "stateMutability": "nonpayable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "bytes32",
                                "name": "role",
                                "type": "bytes32"
                            },
                            {
                                "internalType": "address",
                                "name": "account",
                                "type": "address"
                            }
                        ],
                        "name": "hasRole",
                        "outputs": [
                            {
                                "internalType": "bool",
                                "name": "",
                                "type": "bool"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "string",
                                "name": "_tokenName",
                                "type": "string"
                            },
                            {
                                "internalType": "string",
                                "name": "_tokenSymbol",
                                "type": "string"
                            },
                            {
                                "internalType": "address",
                                "name": "_vault",
                                "type": "address"
                            },
                            {
                                "internalType": "address",
                                "name": "_pool",
                                "type": "address"
                            }
                        ],
                        "name": "initialize",
                        "outputs": [],
                        "stateMutability": "nonpayable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "owner",
                                "type": "address"
                            },
                            {
                                "internalType": "address",
                                "name": "operator",
                                "type": "address"
                            }
                        ],
                        "name": "isApprovedForAll",
                        "outputs": [
                            {
                                "internalType": "bool",
                                "name": "",
                                "type": "bool"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "forwarder",
                                "type": "address"
                            }
                        ],
                        "name": "isTrustedForwarder",
                        "outputs": [
                            {
                                "internalType": "bool",
                                "name": "",
                                "type": "bool"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "bytes32",
                                "name": "commit",
                                "type": "bytes32"
                            }
                        ],
                        "name": "makeCommit",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "name",
                        "outputs": [
                            {
                                "internalType": "string",
                                "name": "",
                                "type": "string"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "ownerOf",
                        "outputs": [
                            {
                                "internalType": "address",
                                "name": "",
                                "type": "address"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "pause",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "paused",
                        "outputs": [
                            {
                                "internalType": "bool",
                                "name": "",
                                "type": "bool"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "pool",
                        "outputs": [
                            {
                                "internalType": "address",
                                "name": "",
                                "type": "address"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "proxiableUUID",
                        "outputs": [
                            {
                                "internalType": "bytes32",
                                "name": "",
                                "type": "bytes32"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "reclaim",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "",
                                "type": "uint256"
                            }
                        ],
                        "name": "recoveryClockOf",
                        "outputs": [
                            {
                                "internalType": "uint256",
                                "name": "",
                                "type": "uint256"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "",
                                "type": "uint256"
                            }
                        ],
                        "name": "recoveryDestinationOf",
                        "outputs": [
                            {
                                "internalType": "address",
                                "name": "",
                                "type": "address"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "",
                                "type": "uint256"
                            }
                        ],
                        "name": "recoveryOf",
                        "outputs": [
                            {
                                "internalType": "address",
                                "name": "",
                                "type": "address"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "bytes16",
                                "name": "fname",
                                "type": "bytes16"
                            },
                            {
                                "internalType": "address",
                                "name": "to",
                                "type": "address"
                            },
                            {
                                "internalType": "bytes32",
                                "name": "secret",
                                "type": "bytes32"
                            },
                            {
                                "internalType": "address",
                                "name": "recovery",
                                "type": "address"
                            }
                        ],
                        "name": "register",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "renew",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "bytes32",
                                "name": "role",
                                "type": "bytes32"
                            },
                            {
                                "internalType": "address",
                                "name": "account",
                                "type": "address"
                            }
                        ],
                        "name": "renounceRole",
                        "outputs": [],
                        "stateMutability": "nonpayable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            },
                            {
                                "internalType": "address",
                                "name": "to",
                                "type": "address"
                            }
                        ],
                        "name": "requestRecovery",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "bytes32",
                                "name": "role",
                                "type": "bytes32"
                            },
                            {
                                "internalType": "address",
                                "name": "account",
                                "type": "address"
                            }
                        ],
                        "name": "revokeRole",
                        "outputs": [],
                        "stateMutability": "nonpayable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "from",
                                "type": "address"
                            },
                            {
                                "internalType": "address",
                                "name": "to",
                                "type": "address"
                            },
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "safeTransferFrom",
                        "outputs": [],
                        "stateMutability": "nonpayable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "from",
                                "type": "address"
                            },
                            {
                                "internalType": "address",
                                "name": "to",
                                "type": "address"
                            },
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            },
                            {
                                "internalType": "bytes",
                                "name": "data",
                                "type": "bytes"
                            }
                        ],
                        "name": "safeTransferFrom",
                        "outputs": [],
                        "stateMutability": "nonpayable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "operator",
                                "type": "address"
                            },
                            {
                                "internalType": "bool",
                                "name": "approved",
                                "type": "bool"
                            }
                        ],
                        "name": "setApprovalForAll",
                        "outputs": [],
                        "stateMutability": "nonpayable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "bytes4",
                                "name": "interfaceId",
                                "type": "bytes4"
                            }
                        ],
                        "name": "supportsInterface",
                        "outputs": [
                            {
                                "internalType": "bool",
                                "name": "",
                                "type": "bool"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "symbol",
                        "outputs": [
                            {
                                "internalType": "string",
                                "name": "",
                                "type": "string"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "bytes32",
                                "name": "",
                                "type": "bytes32"
                            }
                        ],
                        "name": "timestampOf",
                        "outputs": [
                            {
                                "internalType": "uint256",
                                "name": "",
                                "type": "uint256"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "tokenURI",
                        "outputs": [
                            {
                                "internalType": "string",
                                "name": "",
                                "type": "string"
                            }
                        ],
                        "stateMutability": "pure",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "from",
                                "type": "address"
                            },
                            {
                                "internalType": "address",
                                "name": "to",
                                "type": "address"
                            },
                            {
                                "internalType": "uint256",
                                "name": "tokenId",
                                "type": "uint256"
                            }
                        ],
                        "name": "transferFrom",
                        "outputs": [],
                        "stateMutability": "nonpayable",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "trustedCaller",
                        "outputs": [
                            {
                                "internalType": "address",
                                "name": "",
                                "type": "address"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "trustedOnly",
                        "outputs": [
                            {
                                "internalType": "uint256",
                                "name": "",
                                "type": "uint256"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "bytes16",
                                "name": "fname",
                                "type": "bytes16"
                            },
                            {
                                "internalType": "address",
                                "name": "to",
                                "type": "address"
                            },
                            {
                                "internalType": "address",
                                "name": "recovery",
                                "type": "address"
                            },
                            {
                                "internalType": "uint256",
                                "name": "inviter",
                                "type": "uint256"
                            },
                            {
                                "internalType": "uint256",
                                "name": "invitee",
                                "type": "uint256"
                            }
                        ],
                        "name": "trustedRegister",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "unpause",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "newImplementation",
                                "type": "address"
                            }
                        ],
                        "name": "upgradeTo",
                        "outputs": [],
                        "stateMutability": "nonpayable",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "address",
                                "name": "newImplementation",
                                "type": "address"
                            },
                            {
                                "internalType": "bytes",
                                "name": "data",
                                "type": "bytes"
                            }
                        ],
                        "name": "upgradeToAndCall",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    },
                    {
                        "inputs": [],
                        "name": "vault",
                        "outputs": [
                            {
                                "internalType": "address",
                                "name": "",
                                "type": "address"
                            }
                        ],
                        "stateMutability": "view",
                        "type": "function"
                    },
                    {
                        "inputs": [
                            {
                                "internalType": "uint256",
                                "name": "amount",
                                "type": "uint256"
                            }
                        ],
                        "name": "withdraw",
                        "outputs": [],
                        "stateMutability": "payable",
                        "type": "function"
                    }
                ]"#;
                Some(name_registry_abi)
            }
        }
    }
}
